package org.jeecg.modules.checkData.equipmentCheckData.mapper;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.jeecg.modules.checkData.equipmentCheckData.dto.MovementAdditiveInfoDTO;
import org.jeecg.modules.checkData.equipmentCheckData.dto.MovementRecordsMasterDTO;
import org.jeecg.modules.checkData.equipmentCheckData.entity.MovementRecordsMaster;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.jeecg.modules.checkData.equipmentCheckData.vo.MovementAdditiveInfoVO;
import org.jeecg.modules.checkData.equipmentCheckData.vo.MovementRecordsMasterVO;

import java.math.BigDecimal;

/**
 * <p>
 * 动静态几何尺寸数据主表	 Mapper 接口
 * </p>
 *
 * @author hkl
 * @since 2023-07-11
 */
public interface MovementRecordsMasterMapper extends BaseMapper<MovementRecordsMaster> {
    /**
     * 动静态几何尺寸列表-分页列表查询
     *
     * @param pageData
     * @param dto
     * @return
     */
    Page<MovementRecordsMasterVO> queryPageList(Page<MovementRecordsMasterVO> pageData, MovementRecordsMasterDTO dto);

    /**
     * 统计钢轨单元数量
     *
     * @param sectionId       区间id
     * @param lightRailId     线路id
     * @param lineAliasId     线别id
     * @param startingMileage 开始里程
     * @param endMileage      结束里程
     * @return
     */
    Integer getRailNums(String sectionId, String lightRailId, String lineAliasId, BigDecimal startingMileage, BigDecimal endMileage);

    /**
     * 统计曲线数量
     *
     * @param sectionId       区间id
     * @param lightRailId     线路id
     * @param lineAliasId     线别id
     * @param startingMileage 开始里程
     * @param endMileage      结束里程
     * @return
     */
    Integer getCurveNums(String sectionId, String lightRailId, String lineAliasId, BigDecimal startingMileage, BigDecimal endMileage);

    /**
     * 统计竖曲线数量
     * @param sectionId 区间id
     * @param lightRailId 线路id
     * @param lineAliasId 线别id
     * @param startingMileage 开始里程
     * @param endMileage 结束里程
     * @return
     */
    Integer getVerticalCurveNum(String sectionId, String lightRailId, String lineAliasId, BigDecimal startingMileage, BigDecimal endMileage);
    /**
     * 统计道岔数量
     *
     * @param sectionId       区间id
     * @param lightRailId     线路id
     * @param lineAliasId     线别id
     * @param startingMileage 开始里程
     * @param endMileage      结束里程
     * @return
     */
    Integer getSwitchNum(String sectionId, String lightRailId, String lineAliasId, BigDecimal startingMileage, BigDecimal endMileage);
    /**
     * 查询添乘仪详情
     *
     * @param pageData
     * @param dto
     * @return
     */
    Page<MovementAdditiveInfoVO> queryPageAdditiveInfoList(Page<MovementAdditiveInfoVO> pageData, MovementAdditiveInfoDTO dto);
}
