package org.jeecg.modules.deviceAsset.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.modules.deviceAsset.dto.RailManagementDTO;
import org.jeecg.modules.deviceAsset.entity.RailManagement;
import org.jeecg.modules.deviceAsset.service.IRailManagementService;
import org.jeecg.modules.deviceAsset.vo.RailManagementVO;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * <p>
 * 资产管理-钢轨管理 前端控制器
 * </p>
 *
 * @author hkl
 * @since 2023-07-06
 */
@RestController
@RequestMapping("/deviceAsset/railManagement")
@Api(tags = "资产管理-钢轨管理")
public class RailManagementController extends JeecgController<RailManagement, IRailManagementService> {


    @AutoLog(value = "资产管理-钢轨管理-分页列表查询")
    @ApiOperation(value = "资产管理-道岔管理-分页列表查询", notes = "资产管理-道岔管理-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<RailManagementVO>> queryPageList(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         RailManagementDTO dto) {
        Page<RailManagementVO> pageData = new Page<>(pageNo, pageSize);
        pageData = this.service.queryPageList(pageData, dto);
        return Result.OK(pageData);
    }


    @AutoLog(value = "资产管理-钢轨管理-编辑")
    @ApiOperation(value = "资产管理-钢轨管理-编辑", notes = "资产管理-钢轨管理-编辑")
    @PostMapping(value = "/edit")
    public Result<String> edit(@RequestBody RailManagement railManagement) {
        this.service.edit(railManagement);

        return Result.OK("编辑成功!");
    }

    @AutoLog(value = "资产管理-钢轨管理-通过id删除")
    @ApiOperation(value = "资产管理-钢轨管理-通过id删除", notes = "资产管理-钢轨管理-通过id删除")
    @GetMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        this.service.removeById(id);
        return Result.OK("删除成功!");
    }

    @AutoLog(value = "资产管理-钢轨管理-批量删除")
    @ApiOperation(value = "资产管理-钢轨管理-批量删除", notes = "资产管理-钢轨管理-批量删除")
    @GetMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.service.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }


    @AutoLog(value = "资产管理-钢轨管理-智能生成")
    @ApiOperation(value = "资产管理-钢轨管理-智能生成", notes = "资产管理-钢轨管理-智能生成")
    @GetMapping(value = "/intelligentGeneration")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lightRailId", value = "线路id")
    })
    public Result<String> intelligentGeneration(String lightRailId) {
        this.service.intelligentGeneration(lightRailId);
        return Result.OK("生成成功");
    }

}
