package org.jeecg.modules.deviceAsset.controller;


import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.modules.deviceAsset.dto.TrackBedManagementDTO;
import org.jeecg.modules.deviceAsset.entity.TrackBedManagement;
import org.jeecg.modules.deviceAsset.service.ITrackBedManagementService;
import org.jeecg.modules.deviceAsset.vo.TrackBedManagementVO;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 资产管理-道床管理 前端控制器
 * </p>
 *
 * @author hkl
 * @since 2023-07-06
 */
@RestController
@RequestMapping("/deviceAsset/trackBedManagement")
@Api(tags = "资产管理-道床管理")
public class TrackBedManagementController extends JeecgController<TrackBedManagement, ITrackBedManagementService> {
    @AutoLog(value = "资产管理-道床管理-分页列表查询")
    @ApiOperation(value = "资产管理-道床管理-分页列表查询", notes = "资产管理-道床管理-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<TrackBedManagementVO>> queryPageList(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             TrackBedManagementDTO dto) {
        Page<TrackBedManagementVO> pageData = new Page<>(pageNo, pageSize);
        pageData = this.service.queryPageList(pageData, dto);
        return Result.OK(pageData);
    }


    @AutoLog(value = "资产管理-道床管理-编辑")
    @ApiOperation(value = "资产管理-道床管理-编辑", notes = "资产管理-道床管理-编辑")
    @PostMapping(value = "/edit")
    public Result<String> edit(@RequestBody TrackBedManagement trackBedManagement) {
        if (ObjectUtil.isEmpty(trackBedManagement.getId())) {
            this.service.save(trackBedManagement);
        } else {
            this.service.updateById(trackBedManagement);
        }

        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "资产管理-道床管理-通过id删除")
    @ApiOperation(value = "资产管理-道床管理-通过id删除", notes = "资产管理-道床管理-通过id删除")
    @GetMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        this.service.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "资产管理-道床管理-批量删除")
    @ApiOperation(value = "资产管理-道床管理-批量删除", notes = "资产管理-道床管理-批量删除")
    @GetMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.service.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    @AutoLog(value = "资产管理-道床管理-查询列表")
    @ApiOperation(value = "资产管理-道床管理-查询列表", notes = "资产管理-道床管理-查询列表")
    @GetMapping(value = "/queryList")
    public Result<List<TrackBedManagement>> queryList(@ApiParam(name = "区间id") String sectionId,
                                                      @ApiParam(name = "线别id") String lineAliasId) {
        List<TrackBedManagement> listResult = this.service.lambdaQuery()
                .eq(ObjectUtil.isNotEmpty(sectionId), TrackBedManagement::getSectionId, sectionId)
                .eq(ObjectUtil.isNotEmpty(lineAliasId), TrackBedManagement::getLineAliasId, lineAliasId)
                .list();
        return Result.OK(listResult);
    }

}
